@@ -104,6 +104,8 @@ void Pipeline_run_pipeline_multithreaded_tasks(void);
 void Pipeline_pipeline_init_no_terms(void);
 void Pipeline_pipeline_init_no_system_term(void);
 void Pipeline_disable_component_from_immediate_system(void);
+void Pipeline_run_w_empty_query(void);
+void Pipeline_run_w_0_src_query(void);
 
 // Testsuite 'SystemMisc'
 void SystemMisc_invalid_not_without_id(void);
@@ -168,6 +170,7 @@ void SystemMisc_system_w_short_notation(void);
 void SystemMisc_update_interval_w_system_init(void);
 void SystemMisc_update_rate_w_system_init(void);
 void SystemMisc_system_w_interval_rate_stop_timer(void);
+void SystemMisc_system_w_rate_filter_self(void);
 void SystemMisc_system_same_interval_same_tick(void);
 void SystemMisc_system_no_id_in_scope(void);
 void SystemMisc_register_callback_after_run(void);
@@ -452,6 +455,8 @@ void Rest_request_commands_garbage_collect(void);
 void Rest_script_error(void);
 void Rest_import_rest_after_mini(void);
 void Rest_get_pipeline_stats_after_delete_system(void);
+void Rest_request_world_summary_before_monitor_sys_run(void);
+void Rest_escape_backslash(void);
 
 // Testsuite 'Metrics'
 void Metrics_member_gauge_1_entity(void);
@@ -905,6 +910,14 @@ bake_test_case Pipeline_testcases[] = {
     {
         "disable_component_from_immediate_system",
         Pipeline_disable_component_from_immediate_system
+    },
+    {
+        "run_w_empty_query",
+        Pipeline_run_w_empty_query
+    },
+    {
+        "run_w_0_src_query",
+        Pipeline_run_w_0_src_query
     }
 };
 
@@ -1157,6 +1170,10 @@ bake_test_case SystemMisc_testcases[] = {
         "system_w_interval_rate_stop_timer",
         SystemMisc_system_w_interval_rate_stop_timer
     },
+    {
+        "system_w_rate_filter_self",
+        SystemMisc_system_w_rate_filter_self
+    },
     {
         "system_same_interval_same_tick",
         SystemMisc_system_same_interval_same_tick
@@ -2183,6 +2200,14 @@ bake_test_case Rest_testcases[] = {
     {
         "get_pipeline_stats_after_delete_system",
         Rest_get_pipeline_stats_after_delete_system
+    },
+    {
+        "request_world_summary_before_monitor_sys_run",
+        Rest_request_world_summary_before_monitor_sys_run
+    },
+    {
+        "escape_backslash",
+        Rest_escape_backslash
     }
 };
 
@@ -2505,14 +2530,14 @@ static bake_test_suite suites[] = {
         "Pipeline",
         NULL,
         NULL,
-        83,
+        85,
         Pipeline_testcases
     },
     {
         "SystemMisc",
         NULL,
         NULL,
-        68,
+        69,
         SystemMisc_testcases
     },
     {
@@ -2635,7 +2660,7 @@ static bake_test_suite suites[] = {
         "Rest",
         NULL,
         NULL,
-        16,
+        18,
         Rest_testcases
     },
     {
