@@ -130,6 +130,56 @@ void Sparse_1_this_sparse(void) {
     ecs_fini(world);
 }
 
+void Sparse_1_this_sparse_simple(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_id(world, ecs_id(Position), EcsSparse);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }},
+        .cache_kind = cache_kind
+    });
+    test_assert(q != NULL);
+
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+
+    ecs_entity_t e1 = ecs_insert(world, ecs_value(Position, {10, 20}));
+    ecs_entity_t e2 = ecs_insert(world, ecs_value(Position, {30, 40}));
+    ecs_entity_t e3 = ecs_insert(world, ecs_value(Position, {50, 60}));
+
+    ecs_iter_t it = ecs_query_iter(world, q);
+    test_bool(true, ecs_query_next(&it));
+    test_int(3, it.count);
+    test_uint(e1, it.entities[0]);
+    {
+        Position *p = ecs_field_at(&it, Position, 0, 0);
+        test_assert(p != NULL);
+        test_int(p->x, 10); test_int(p->y, 20);
+    }
+
+    test_uint(e2, it.entities[1]);
+    {
+        Position *p = ecs_field_at(&it, Position, 0, 1);
+        test_assert(p != NULL);
+        test_int(p->x, 30); test_int(p->y, 40);
+    }
+
+    test_uint(e3, it.entities[2]);
+    {
+        Position *p = ecs_field_at(&it, Position, 0, 2);
+        test_assert(p != NULL);
+        test_int(p->x, 50); test_int(p->y, 60);
+    }
+
+    test_bool(false, ecs_query_next(&it));
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
 void Sparse_1_this_sparse_none(void) {
     ecs_world_t *world = ecs_mini();
 
@@ -450,6 +500,80 @@ void Sparse_1_var_sparse_written_none(void) {
     ecs_fini(world);
 }
 
+void Sparse_2_sparse_simple(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_COMPONENT(world, Velocity);
+
+    ecs_add_id(world, ecs_id(Position), EcsSparse);
+    ecs_add_id(world, ecs_id(Velocity), EcsSparse);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ .id = ecs_id(Position) }, { .id = ecs_id(Velocity) }},
+        .cache_kind = cache_kind
+    });
+    test_assert(q != NULL);
+
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+    test_bool(true, !!(q->row_fields & (1llu << 1)));
+
+    ecs_entity_t e1 = ecs_insert(world, 
+        ecs_value(Position, {10, 20}),
+        ecs_value(Velocity, {1,  2}));
+    ecs_entity_t e2 = ecs_insert(world, 
+        ecs_value(Position, {30, 40}),
+        ecs_value(Velocity, {3,  4}));
+    ecs_entity_t e3 = ecs_insert(world, 
+        ecs_value(Position, {50, 60}),
+        ecs_value(Velocity, {5,  6}));
+
+    ecs_iter_t it = ecs_query_iter(world, q);
+    test_bool(true, ecs_query_next(&it));
+    test_int(3, it.count);
+    test_uint(e1, it.entities[0]);
+    {
+        Position *p = ecs_field_at(&it, Position, 0, 0);
+        test_assert(p != NULL);
+        test_int(p->x, 10); test_int(p->y, 20);
+    }
+    {
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 0);
+        test_assert(v != NULL);
+        test_int(v->x, 1); test_int(v->y, 2);
+    }
+
+    test_uint(e2, it.entities[1]);
+    {
+        Position *p = ecs_field_at(&it, Position, 0, 1);
+        test_assert(p != NULL);
+        test_int(p->x, 30); test_int(p->y, 40);
+    }
+    {
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 1);
+        test_assert(v != NULL);
+        test_int(v->x, 3); test_int(v->y, 4);
+    }
+
+    test_uint(e3, it.entities[2]);
+    {
+        Position *p = ecs_field_at(&it, Position, 0, 2);
+        test_assert(p != NULL);
+        test_int(p->x, 50); test_int(p->y, 60);
+    }
+    {
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 2);
+        test_assert(v != NULL);
+        test_int(v->x, 5); test_int(v->y, 6);
+    }
+
+    test_bool(false, ecs_query_next(&it));
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
 void Sparse_2_sparse(void) {
     ecs_world_t *world = ecs_mini();
 
